function [f1,f2,f3,f4]=RemoteNodeForce_aniso(x1,x2,x3,x4,bp,b,a,C,rtol,flag_rem)
% this calculates the forces between dislocation nodes by numerical integration
%inputs: endpoints of first dislocation segment starting at x1 ending at x2 with burgers vector bp
%        endpoints of second dislocation segment starting at x3 ending at x4 with burgers vector b
%        core paramter a
%
%outputs: f1,f2,f3,f4 is the force on nodes located at x1, x2, x3, x4
%respectively
%
%Two segments must be remote, but can be connected
%
%default tolerance
if(~exist('rtol'))
    rtol = 1e-4;
end
%
%sift special cases 
%
%default flag, 0(free)
if(~exist('flag_rem'))
    flag_rem = [0 0 0 0];
end
%Four nodes are all fixed
if(min(flag_rem == 7))
    f1=zeros(1,3);
    f2=zeros(1,3);
    f3=zeros(1,3);
    f4=zeros(1,3);
    return;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Treat collinear cases separately
t1 = (x2-x1)/norm(x2-x1);
col_tol = 1e-6;
d3 = norm((x3-x2)-dot((x3-x2),t1)*t1);
d4 = norm((x4-x2)-dot((x4-x2),t1)*t1);
if max(d3,d4)<col_tol
    %collinear case
    if dot((x3-x2),t1) <= dot((x4-x2),t1)
        [f1,f2,f3,f4] = CollinearNodeForce(x1,x2,x3,x4,bp,b,a,C);
        return;
    else
        [f1,f2,f4,f3] = CollinearNodeForce(x1,x2,x4,x3,bp,b,a,C);
        return;
    end
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%All other cases
[f1,f2,f3,f4]=RemoteNodeForce_aniso_core(x1,x2,x3,x4,bp,b,a,C,rtol,flag_rem); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function [f1,f2,f3,f4]=RemoteNodeForce_aniso_core(x1,x2,x3,x4,bp,b,a,C,rtol,flag_rem)

% this calculates the forces between dislocation nodes by numerical integration
%inputs: endpoints of first dislocation segment starting at x1 ending at x2 with burgers vector bp
%        endpoints of second dislocation segment starting at x3 ending at x4 with burgers vector b
%        core paramter a
%
%outputs: f1,f2,f3,f4 is the force on nodes located at x1, x2, x3, x4
%respectively
%
%   
%default flag, 0(free)
if(~exist('flag_rem'))
    flag_rem = [0 0 0 0];
end

N_ini = 2;%default increment value
dsc_rule = 'midpoint';
%dsc_rule = 'endpoint';
%dsc_rule = 'trapezoid';

N_old = N_ini; N_new = N_old; f_int12_old = [];  f_int34_old = [];
[f11,f21,f_int12]=RemoteNodeForce_aniso_int(x1,x2,x3,x4,bp,b,a,C,N_new,flag_rem(1:2),dsc_rule,f_int12_old);
[f31,f41,f_int34]=RemoteNodeForce_aniso_int(x3,x4,x1,x2,b,bp,a,C,N_new,flag_rem(3:4),dsc_rule,f_int34_old);

myerror = 10;
while myerror > rtol 
    f10 = f11; f20 = f21; f30 = f31; f40 = f41;
    N_new = N_new*N_ini;
    f_int12_old = f_int12; f_int34_old = f_int34;
    [f11,f21,f_int12]=RemoteNodeForce_aniso_int(x1,x2,x3,x4,bp,b,a,C,N_new,flag_rem(1:2),dsc_rule,f_int12_old);
    [f31,f41,f_int34]=RemoteNodeForce_aniso_int(x3,x4,x1,x2,b,bp,a,C,N_new,flag_rem(3:4),dsc_rule,f_int34_old);
    %
    myerror = max([Error_v(f10,f11),Error_v(f20,f21),Error_v(f30,f31),Error_v(f40,f41)]);
    %
    if N_new > 1e4
        disp(sprintf('Warning, N_int exceeds 1e4'));
        f1 = f11; f2 = f21; f3 = f31; f4 = f41;
        return
    end
 
%diagnosis
%if N_new > 200
%     ratio = norm(x3-x4)/norm(x2-x1);
%     ratio_max = max(ratio,1/ratio);
% disp(sprintf('RemoteNodeForce_aniso N_new = %d myerror = %e',N_new,myerror));
% disp(sprintf('  N_new = %d   x1,x2,x3,x4 = [ %g %g %g   %g %g %g   %g %g %g   %g %g %g ]',...
%                  N_new, x1(1),x1(2),x1(3),  x2(1),x2(2),x2(3), x3(1),x3(2),x3(3), x4(1),x4(2),x4(3)));
% disp(sprintf('  N_new = %d   flag_rem = [ %d %d %d %d ]',...
%                      N_new, flag_rem(1),flag_rem(2),flag_rem(3),flag_rem(4)));
% disp(sprintf('The segment length ratio is %d',ratio_max));
%end

end
           
%
%output
f1 = f11; f2 = f21; f3 = f31; f4 = f41;
%
% plot intermediate results
%figure(1);
%plot([1:length(f_int12(:,1))],f_int12,'.-',[1:length(f_int34(:,1))],f_int34,'.-');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    
function [f1,f2,f_int]=RemoteNodeForce_aniso_int(x1,x2,x3,x4,bp,b,a,C,N,flag_rem,dsc_rule,f_int_old);
% calculate the remote nodal force as a function of integration numbers
% integration points are always evenly distribution on segment 1-2
% depending on number of integration points N
if nargin < 12
    flag_rem = [0,0];
end
%
flag1 = flag_rem(1);
flag2 = flag_rem(2);
%
%initializing f's
f1=zeros(1,3);
f2=zeros(1,3);
%    
if flag1 == 7  && flag2 == 7
    f1=zeros(1,3);f2=zeros(1,3);
    int_f1=zeros(N,3);
    int_f2=zeros(N,3);
    f_int = [int_f1; int_f2];
    return
end

%calculate f1 and f2
if strcmp(dsc_rule,'midpoint')
    alpha = ([1:N]-1/2)/N; 
    weight = ones(size(alpha))/N;
elseif strcmp(dsc_rule,'endpoint')
    alpha = ([1:N-1])/N;
    weight = ones(size(alpha))/N;
    if N>2
        weight(1)   = 1.5/N;
        weight(end) = 1.5/N;
    end
elseif strcmp(dsc_rule,'trapezoid')
    alpha = ([0:N])/N;
    weight = ones(size(alpha))/N;
    weight(1)   = 0.5/N;
    weight(end) = 0.5/N;
else
    disp('Error: unknown dsc_rule!');
    return;
end

seg_12_x = x1(1) + (x2(1)-x1(1))*alpha;
seg_12_y = x1(2) + (x2(2)-x1(2))*alpha;
seg_12_z = x1(3) + (x2(3)-x1(3))*alpha;
    
xi_12=(x2-x1)/norm(x2-x1);

int_f1=zeros(length(alpha),3); int_f2=zeros(length(alpha),3);
for i=1:length(alpha)
    P=[seg_12_x(i),seg_12_y(i),seg_12_z(i)];
    % compute stress using matrix_WS method
    f_34=stress_seg_WS(x3,x4,P,b,C,'m',a);
    % update PK force (times weighting function) only if node is free
    if(flag1 == 0)
        int_f1(i,:)=cross((f_34*bp')',xi_12)*norm(P-x2)/norm(x1-x2);
    end
    if(flag2 == 0)
        int_f2(i,:)=cross((f_34*bp')',xi_12)*norm(P-x1)/norm(x1-x2);
    end
end

% sum over integration points to get nodal force
for i=1:3
    for j=1:length(alpha)
           f1(i)=f1(i)+int_f1(j,i)*norm(x2-x1)*weight(j);
           f2(i)=f2(i)+int_f2(j,i)*norm(x2-x1)*weight(j);
    end
end
% return intermediate results
f_int = [int_f1; int_f2];
return;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% %calculating the nodal forces of two connected segments X1-X2-X3
% %
% function [F1,F2,F3] = RemoteNodeForce_aniso_connect(X1,X2,X3,bp,b,a,c11,c12,c44,rtol)
% %
% %calculate F1,F2 first
% if norm(X3-X2)<=a
%     [f1,f2,f3,f4]=RemoteNodeForce_aniso_core(X1,X2,X2,X3,bp,b,a,c11,c12,c44,rtol);
%     F1 = f1;F2 =f2; F3 = f4;return;
% else
%     N_a = round(norm(X3-X2)/a);
%     N_a = N_a +(norm(X3-X2)-N_a*a>0);
% end
% %
% xi_2 = (X3-X2)/norm(X3-X2);
% %initializing
% F1 = zeros(1,3); F2 = zeros(1,3); F3 = zeros(1,3);
% for i = 1:N_a-1
%     x3 = X2 + a*(i-1)*xi_2; x4 = x3+a*xi_2;
%     [f1,f2,f3,f4]=RemoteNodeForce_aniso_core(X1,X2,x3,x4,bp,b,a,c11,c12,c44,rtol);
%     F1 = F1+f1; F2 = F2+f2;
% end
% % [f1,f2,f3,f4]=RemoteNodeForce_aniso_core(X1,X2,x4,X3,bp,b,a,c11,c12,c44,rtol);
% % F1 = F1+f1; F2 = F2+f2;
% %
% %calculating F3
% if norm(X2-X1)<=a
%     [f1,f2,f3,f4]=RemoteNodeForce_aniso_core(X1,X2,X2,X3,bp,b,a,c11,c12,c44,rtol);
%     F1 = f1;F2 =f2; F3 = f4;return;
% else
%     N_a = round(norm(X2-X1)/a)+1;
%     N_a = N_a +(norm(X2-X1)-N_a*a>0);
% end
% %
% xi_1 = (X2-X1)/norm(X2-X1);
% %initializing
% for i = 1:N_a-1
%     x1 = X1 + a*(i-1)*xi_1;x2 = x1+a*xi_1;  
%     [f1,f2,f3,f4]=RemoteNodeForce_aniso_core(x1,x2,X2,X3,bp,b,a,c11,c12,c44,rtol);
%     F3 = F3+f4;
% end
% % [f1,f2,f3,f4]=RemoteNodeForce_aniso_core(x2,X2,X2,X3,bp,b,a,c11,c12,c44,rtol);
% % F3 = F3+f4;