function sig_c = critical_stress(L, charact, sig_0, dsig)
%calculate critical stress to activate a Frank Read Source
%real slip system is not involved

%input
%L: length of dislocation segment (unit: angstrom)
%charact: charcateristic of the dislocation segment, pure edge or screw
%sig_0: starting magnitude of external stress
%dsig: increment of external stress

global Bscrew Bedge Beclimb Bline

%===================================================
%Part 1 Basic Parameters

%cut off radius
%a=lmin/sqrt(3)*0.5;
%a = 0.1;
a = 10;

%number of steps for each cycle
totalsteps = 10;

%maximum time step size
%dt0=1e7; 
dt0 = 1e3;

%whether it is a close dislocation loop, "1" for Yes
loop = 1;

%whether the initial dislocation is perturbed (little bowed) "1" = Yes
perturb_config = 1;

%Lattice constant
%Tungsten,unit:angstrom
la = 3.165*sqrt(3)/2;

%========================================
%Part 2 Initial dislocation configuration setup

%aspect ratio for the loop = height/segment length
ha = 4;
%number of free segments
N_seg = 10;
d_seg = L/N_seg;
%
if loop ==1
    y = [-L/2, L/2, L/2, -L/2:d_seg:(L/2-d_seg) ];
    z = [0 0 -ha*L -ha*L zeros(1,length(y)-4)];
    flag = [7 7 7 7 zeros(1,length(y)-4)];%7=fixed;0=free
    links = zeros(length(y),8);
    links(:,1) = [1, 5:length(y),2,3,4]';
    links(:,2) = [5:length(y),2,3,4,1]';
    printnode=N_seg/2+4;
    %printnode = 1;
else
    y = [L/2,-L/2:d_seg:(L/2-d_seg)];
    z = zeros(1,length(y));
    flag = [7,7,zeros(1,length(y)-2)];
    links = zeros(length(y)-1,8);
    links(:,1) = [2:length(y)]';
    links(:,2) = [3:length(y),1]';
    printnode = 2 + N_seg/2;
end
%
if perturb_config == 1
    A = L/50;
    x = A*cos(y*pi/L);
else
    x = zeros(1, length(y));
end
%    
rn = [x;y;z;flag]';
%
if charact == 'e'
    %Edge dislocation segment
    links(:,3) = 1*la;
    links(:,8) = 1;
    appstr0 = [0  0  -1; 0 0 0; -1 0 0];
    %
elseif charact == 's'
%Screw Dislocation segment
    links(:,4) = 1*la;
    links(:,8) = 1;
    appstr0 = [0  0  0; 0 0 -1; 0 -1 0];
    %
elseif charact == 'm'
%mixed Dislocation
    links(:,3) = 1*la;
    links(:,4) = 1*la;
    links(:,8) = 1;
    appstr0 = [0 0 -1; 0 0 -1; -1 -1 0]*la;
    %
end

%====================================
%Part 3 Simulation Parameters

%Mo
%c44=11.0;
%c11=46.0;
%c12=17.6;

%W
c11=52.1;
c12=20.1;
c44=16.0001;%(perturbed by 1e-4)

%construct elastic constant tensor
C = cubicelasticconst(c11,c12,c44);
%
%tungsten
MU = 16.0;
NU = 0.278;
%
%Topological constraints
maxconnections=8;
lmax = L;
lmin = L/20;
areamin=lmin*lmin*sin(60/180*pi)*0.5; 
areamax=20*areamin;


%Core energy
%Ec = MU/(4*pi)*log(a/0.1);
Ec = 0;

%Mobility law (for most cases)
mobility='mobfcc1';

%Drag (Mobility) parameters
Bscrew=1e0;
Bedge=1e0;
Beclimb=1e2;
Bline=1.0e-4*min(Bscrew,Bedge);

integrator='int_trapezoid';
%rann = 0.001;       
%rntol = 0.01*L;
rann = 0.5*a;       
rntol = 0.5*rann;

%no remesh enabled
doremesh=0;
docollision=0;
doseparation=0;
elasticinteraction=1; %line tension model when this is zero
plotfreq=1;       
plim=5*max(max(rn));

viewangle=[0 90];
printfreq=1;      

rmax=100;

%==========================================
%Part 4 Applying external stress

if sig_0 == 0
    appstr_mag = dsig;
else
    appstr_mag = sig_0;
end
%
if loop ==1
    MAX_old = max(abs(rn(5:end,1)));
else
    MAX_old = max(abs(rn(3:end,1)));
end
%  
appliedstress = appstr0*appstr_mag;
disp(sprintf('applied stress magnitude = %e  totalsteps = %d',appstr_mag,totalsteps));
dd3d
filename = sprintf('data/stress_%g_%g_init.mat',L,appstr_mag);
save(filename);
%
if loop ==1
    MAX_new = max(abs(rn(5:end,1)));
else
    MAX_new = max(abs(rn(3:end,1)));
end
%
%
%
%initializing x-component of velocity
v0 = vn(printnode,1);
vf = vn(printnode,1);
%
cycle = 1;
while MAX_new < L%/2
    if MAX_new > MAX_old && vf > v0*0.1%rate constraint
        MAX_old = MAX_new;
        appliedstress = appstr0*appstr_mag;
        disp(sprintf('applied stress magnitude = %e  totalsteps = %d',appstr_mag,totalsteps));
        %plim=5*max(max(rn));
        dd3d
        filename = sprintf('data/stress_%g_%g_cycle%03d.mat',L,appstr_mag,cycle);
        save(filename);
        vf = vn(printnode,1);
        cycle = cycle + 1;
        if loop ==1
            MAX_new = max(abs(rn(5:end,1)));
        else
            MAX_new = max(abs(rn(3:end,1)));
        end
        %MAX_new
    else
        MAX_old = MAX_new;
        % save configuration
        filename = sprintf('data/stress_%g_%g_final.mat',L,appstr_mag);
        save(filename);
        % about to apply new stress
        appstr_mag = appstr_mag + dsig;
        appliedstress = appstr0*appstr_mag;%newly added
        cycle = 1;
        disp(sprintf('applied stress magnitude = %e  totalsteps = %d',appstr_mag,totalsteps));
        %plim=5*max(max(rn));
        dd3d
        if loop ==1
            MAX_new = max(abs(rn(5:end,1)));
        else
            MAX_new = max(abs(rn(3:end,1)));
        end
        v0 = vn(printnode,1);
        vf = vn(printnode,1);
    end
end
%
sig_c = max(max(abs(appliedstress)));