function sig_c = critical_stress_aniso(L, charact, sig_0, dsig)
%calculate critical stress to activate a Frank Read Source
%real slip system is involved
%"aniso" corresponds to BCC iron specifically
%closed dislocation loop only; configuration slightly perturbed initially
%i.e. loop = 1; perturb_config = 1;

%input
%L: length of dislocation segment (unit: angstrom)
%charact: charcateristic of the dislocation segment, pure edge or screw
%sig_0: starting magnitude of external stress
%dsig: increment of external stress
%
global Bscrew Bedge Beclimb Bline

%=================================================
%Part 1 Basic Parameters

%cut off radius
%a=lmin/sqrt(3)*0.5;
%a = 0.1;
a = 10;

%number of steps for each cycle
totalsteps = 10;

%maximum time step size
%dt0=1e7; 
dt0 = 1e3;

%Lattice constant
%Iron,unit:angstrom
la = 2.867;

%slip system
n = [1 1 0];
b = [-1 1 1]/2*la;

%the case in critical_stress.m
% n = [0 0 1];
% b = [1 0 0];

%=================================================
%Part 2 Initial dislocation configuration setup

%aspect ratio for the loop = height/segment length
ha = 4;
%number of free segments
N_seg = 10;
d_seg = L/N_seg;

%initializing
n0 = n/norm(n);
b0 = b/norm(b);

%coordinates in (x,y,z)
x = [-L/2, L/2, L/2, -L/2:d_seg:(L/2-d_seg) ]';
A = L/50;%perturbation in expansion direction
y = A*cos(x*pi/L);
z = [0 0 -ha*L -ha*L zeros(1,length(y)-4)]';
flag = [7 7 7 7 zeros(1,length(y)-4)];%7=fixed;0=free
printnode=N_seg/2+4;

%    
if charact == 'e'
    xi = cross(n0,b0); %dislocation line orientation
    m = b0;
    %define (x,y,z) // (xi,b0,n)
    %define rn   
    rn = [[x,y,z]*[xi;m;n0],flag'];
elseif charact == 's'
    xi = b0;
    m = cross(xi,n0);
    %define (x,y,z) // (xi,m,n)
    %define rn
    rn = [[x,y,z]*[xi;m;n0],flag'];
end

%
%define links    
links = zeros(length(y),8);    
links(:,1) = [1, 5:length(y),2,3,4]';
links(:,2) = [5:length(y),2,3,4,1]';    
for i = 1:3
    links(:,2+i) = b(i);
    links(:,5+i) = n(i);
end
%
appstr0 = -n'*b;
appstr0 = appstr0/norm(appstr0);
%appstr*b = -n

%====================================
%Part 3 Simulation Parameters

%Fe
c11 = 24.2;
c12 = 14.65;
c44 = 11.2;
MU = 7.30;
NU = 0.419;
%
%W
% c11=52.1;
% c12=20.1;
% c44=16.0001;
% MU = 16.0;
% NU = 0.278;

%construct elastic constant tensor
C = cubicelasticconst(c11,c12,c44);

%Topological constraints
maxconnections=8;
lmax = L;
lmin = L/10;
areamin=lmin*lmin*sin(60/180*pi)*0.5; 
areamax=20*areamin;

%Core energy
%Ec = MU/(4*pi)*log(a/0.1);
Ec = 0;

%Mobility law (for most cases)
mobility='mobfcc1';

%Drag (Mobility) parameters
Bscrew=1e0;
Bedge=1e0;
Beclimb=1e2;
Bline=1.0e-4*min(Bscrew,Bedge);

integrator='int_trapezoid';
rann = 0.001;       
rntol = 0.001*L;
%rann = 0.5*a;       
%rntol = 0.5*rann;    

doremesh=0;
docollision=0;
doseparation=0;
elasticinteraction=1; %line tension model when this is zero
plotfreq=1;       
plim=5*max(max(rn));

viewangle = [180 0];
printfreq=1;   
rmax=100;

%==========================================
%Part 4 Applying external stress
%
if sig_0 == 0
    appstr_mag = dsig;
else
    appstr_mag = sig_0;
end
%
MAX_old = max(abs(rn(5:end,1:3)*m'));
% 
%
appliedstress = appstr0*appstr_mag;
disp(sprintf('applied stress magnitude = %e  totalsteps = %d',appstr_mag,totalsteps));
dd3d
filename = sprintf('data/stress_%g_%g_init.mat',L,appstr_mag);
save(filename);
vf = norm(vn(printnode,:));
%
MAX_new = max(abs(rn(5:end,1:3)*m'));
%
%
cycle = 1;
while MAX_new < L  
    if MAX_new > MAX_old && vf*dt0 > 1e-3
        MAX_old = MAX_new;
        appliedstress = appstr0*appstr_mag;
        disp(sprintf('applied stress magnitude = %e  totalsteps = %d',appstr_mag,totalsteps));
        %plim=5*max(max(rn));
        dd3d
        filename = sprintf('data/stress_%g_%g_cycle%03d.mat',L,appstr_mag,cycle);
        save(filename);
        cycle = cycle + 1;
        MAX_new = max(abs(rn(5:end,1:3)*m'));
        vf = norm(vn(printnode,:));
    else
        MAX_old = MAX_new;
        % save configuration
        filename = sprintf('data/stress_%g_%g_final.mat',L,appstr_mag);
        save(filename);
        % about to apply new stress
        appstr_mag = appstr_mag + dsig;
        appliedstress = appstr0*appstr_mag;%newly added
        cycle = 1;
        disp(sprintf('applied stress magnitude = %e  totalsteps = %d',appstr_mag,totalsteps));
        %plim=5*max(max(rn));
        dd3d
        MAX_new = max(abs(rn(5:end,1:3)*m'));
        vf = norm(vn(printnode,:));
    end
end
%
sig_c = appstr_mag;

