function fac_co_stress = fac_co(t,b,C,meth,a,Nint)
%angular factor used for computing collinear stresses
%Based on Willis-Steeds-Lothe formula

%t: (1x3) unit row vector, indicating dislocation orientation
%b:(1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: method used, 'i' or 'm'
%a: dislocation core radius
%Nint is only valid when meth = 'i'

%meth and a must be specified

if meth == 'm'
    fac_co_stress = fac_co_m(t,b,C,a);
elseif meth == 'i' && (~exist('Nint'))
    fac_co_stress = fac_co_i(t,b,C,a);
else
    fac_co_stress = fac_co_i(t,b,C,a, Nint);
end

%=============================================

function fac_co_stress = fac_co_m(t,b,C,a)
%using matrix formalism

%Step 1 Prepare m-n-t and intermediate matrices
[m,n] = rhcoord(t);
[Q,B,S]=stroh_m(t,C);
[Q_p,B_p,S_p] = stroh_p(t,n,b,C,'m');

%initializing
z3 = zeros(3);
nn = z3; nt=z3; nm=z3;

%defining ( )'s
for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                nt(j,k)=nt(j,k)+n(i)*C(i,j,k,l)*t(l);
                nm(j,k)=nm(j,k)+n(i)*C(i,j,k,l)*m(l);
            end
        end
    end
end
nn_1=inv(nn);

%define epsilon
epsilon = zeros(3,3,3);
epsilon(1,2,3)=1; epsilon(2,3,1)=1; epsilon(3,1,2)=1;
epsilon(1,3,2)=-1;epsilon(2,1,3)=-1;epsilon(3,2,1)=-1;

%Step 2 Compute Strain
strain_fac = z3;
M0 = -nn_1*nt*Q + nn_1*nm*Q_p + nn_1*(S_p)';
for p = 1:3
    for s = 1:3
        for j = 1:3
            for q = 1:3
                for i = 1:3
                    for k = 1:3
                        for l = 1:3
                            strain_fac(p,s) = strain_fac(p,s) + 1/4/pi*epsilon(j,s,q)...
                                *b(i)*C(i,j,k,l)*t(q)*(t(l)*Q(p,k)-m(l)*Q_p(p,k)+n(l)*M0(p,k));
                        end
                    end
                end
            end
        end
    end
end

%Step 3 Compute collinear stress
fac_co_stress = z3;
for i = 1:3
    for j = 1:3
        for k = 1:3
            for l = 1:3
                fac_co_stress(i,j) = fac_co_stress(i,j) + C(i,j,k,l)*strain_fac(k,l);
            end
        end
    end
end

%================================================

function fac_co_stress = fac_co_i(t,b,C,a, Nint)
%using integral formalism

%Step 1 Prepare integrand

[M,N] = rhcoord(t);

%define epsilon
epsilon = zeros(3,3,3);
epsilon(1,2,3)=1;epsilon(2,3,1)=1;epsilon(3,1,2)=1;
epsilon(1,3,2)=-1;epsilon(2,1,3)=-1;epsilon(3,2,1)=-1;

%default N_int
if(~exist('Nint'))
    N_int = 101;
end

dphi=2*pi/N_int;
strain_i = zeros(9, N_int+1);

z3 = zeros(3);

%integrand
for k = 1:N_int+1
    zz = z3;zz_1 = z3;
    tz = z3;zt = z3;
    phi = (k-1)*dphi;
    z = N*cos(phi)-M*sin(phi);
    for i =1:3
        for j = 1:3
            for q = 1:3
                for l = 1:3
                    zz(j,q) = zz(j,q)+z(i)*C(i,j,q,l)*z(l);
                    tz(j,q) = tz(j,q)+t(i)*C(i,j,q,l)*z(l);
                    zt(j,q) = zt(j,q)+z(i)*C(i,j,q,l)*t(l);
                end
            end
        end
    end
    zz_1 = inv(zz);
    M0 = zz_1*(tz+zt)*zz_1;
    for i = 1:3
        for p = 1:3
            for j =1:3
                for w = 1:3
                    for q = 1:3
                        for r = 1:3
                            for s = 1:3
                                strain_i(3*(i-1)+p,k) = strain_i(3*(i-1)+p,k) - 1/8/pi^2*epsilon(p,j,w)*...
                                    b(q)*C(w,q,r,s)*t(j)*(t(s)*zz_1(i,r)-z(s)*M0(i,r));
                            end
                        end
                    end
                end
            end
        end
    end
end

%Step 2 Integrate and obtain Collinear Stress
fac_co_stress = z3;
phi = 0:dphi:2*pi;
for i = 1:3
    for j = 1:3
        for k = 1:3
            for l = 1:3
                fac_co_stress(i,j) = fac_co_stress(i,j) + C(i,j,k,l)*trapz(phi',strain_i(3*(k-1)+l,:));
            end
        end
    end
end