function [fseg]=segforcevec_quad(C,a,rn,links,sigext,linkid,elasticinteraction,Ec);
%compute nodal driving force of dislocation network by stress formula

%input: (vectorized version)
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%a: dislocation core radius, default value is zero
%rn: nodal position
%links: dislocation segments ([connectivity, Burgers vector])
%(id1,id2,bx,by,bz,nx,ny,nz)
%sigext: external applied stress

%segment contribution to node force
%no image stress included

%NMAX: max number of nodes[NMAX,m]=size(rn);
[NMAX,m]=size(rn);
if(m~=4)
    disp('rn should have 4 columns!');
    return;
end
[LINKMAX,m]=size(links);

%construct segment list
segments=constructsegmentlist(rn,links);
if linkid==0
    %PK force due to applied stress
    fpk=pkforcevec(sigext,segments);
    
    %self force due to self stress
    [fs0,fs1]=selfforcevec(C,a,segments,elasticinteraction,Ec);
    
    if elasticinteraction == 1
        %remote force due to remote stress
        [fr0,fr1]=remoteforcevec(C,a,segments,0);
        fseg=[fpk, fpk]*0.5+[fs0, fs1]+[fr0, fr1];

    elseif elasticinteraction == 0
        %add force contributions together
        fseg=[fpk, fpk]*0.5+[fs0, fs1];
    end
else
    %PK force due to applied stress
    fpk=pkforcevec(sigext,segments(linkid,:));

    %self force due to self stress
    [fs0,fs1]=selfforcevec(C,a,segments(linkid,:),elasticinteraction,Ec);

    if elasticinteraction == 1
        %remote force due to remote stress
        %[fr0,fr1]=remoteforcevec(MU,NU,a,segments,linkid);
        [fr0,fr1]=remoteforcevec(C,a,segments,linkid);
        %add force contributions together
        fseg=[fpk, fpk]*0.5+[fs0, fs1]+[fr0, fr1];

    elseif elasticinteraction == 0
        fseg=[fpk, fpk]*0.5+[fs0, fs1];
    end    
end

%=========================================================

function segments=constructsegmentlist(rn,links)

[LINKMAX,m]=size(links);

segments=zeros(LINKMAX,16);
%15,16th column is seg_flag (1-fixed,1;2-fixed,2;free,0;1,2 fixed-12)
nseg=0;
for i=1:LINKMAX,
    n0=links(i,1);
    n1=links(i,2);
    if((n0~=0)&(n1~=0))
        nseg=nseg+1;
        %
        segments(nseg,:)=[links(i,1:5),rn(n0,1:3),rn(n1,1:3),links(i,6:8),rn(n0,4),rn(n1,4)];
    end
end
segments=segments(1:nseg,:);

%==========================================================

function f=pkforcevec(sigext,segments)
%nodal force on dislocation segments due to applied stress sigext (external)
%(vectorized version)
%format of "segments" array:
% (n0,n1,bx,by,bz,x0,y0,z0,x1,y1,z1,nx,ny,nz)
[nseg,m]=size(segments);
f=zeros(nseg,3);

b=segments(:,3:5);
sigb=sigext*b';
r0=segments(:,6:8);
r1=segments(:,9:11);
r01=r1-r0;

for i=1:nseg,
    f(i,:)=cross(sigb(:,i)',r01(i,:));
end

%========================================================

function [f1,f2]=selfforcevec(C,a,segments,elasticinteraction,Ec)
%
%core energy term enabled, which only works for isotropic media
%
%self force (due to self stress)
%(vectorized version)
%format of segments array:
% (n0,n1,bx,by,bz,x0,y0,z0,x1,y1,z1)
%
%
[m,n] = size(segments);
f1 = zeros(m,3);
f2 = zeros(m,3);
%
%default method is matrix formalism
meth = 'm';
%
if elasticinteraction == 1
    for i = 1:m
        A = segments(i, 6:8);
        B = segments(i, 9:11);
        b = segments(i, 3:5);              
        [fs1,fs2] = seg_force_self(A,B,b,C,meth,a);
        for j = 1:3
            f1(i,j) = fs1(j);
            f2(i,j) = fs2(j);
        end
    end
elseif elasticinteraction == 0
    return
end
%
%To include Ecore for isotropic media
%for cubic materials
c11 = C(1,1,1,1);c12 = C(2,2,1,1);c44 = C(1,2,2,1);
H=2*c44+c12-c11;
tol = 1e-3;
if H < tol
    mu=c44; nu=c12/(c11+c12);
    [f10,f20]=selfforcevec_core(mu,nu,a,segments,elasticinteraction,Ec);
    f1 = f1+f10;
    f2 = f2+f20;
end

%===========================================================
function [f0,f1]=remoteforcevec(C,a,segments,linkid)
%nodal force on dislocation segment 01 due to another segment 23
%format of segments array:
% (n0,n1,bx,by,bz,x0,y0,z0,x1,y1,z1,,nx,ny,nz)
%
tol = 1e-3;%tolerance for numerical integration

lseg=size(segments,1);
if linkid==0
    f0=zeros(lseg,3);
    f1=zeros(lseg,3);
    k=1;
    for i=1:lseg
        %disp(sprintf('remoteforcevec: i = %d/%d',i,lseg));
        b1=segments(i,3:5);
        x1=segments(i,6:8);
        x2=segments(i,9:11);
        %index of node 1 and 2
        n1 = segments(i,1);
        n2 = segments(i,2);
        flag1 = segments(i,15);
        flag2 = segments(i,16);
        for j=(i+1):lseg
            %disp(sprintf('remoteforcevec: j = %d/%d',j,lseg));
            b2=segments(j,3:5);
            x3=segments(j,6:8);
            x4=segments(j,9:11);
            %index of node 1 and 2
            n3 = segments(j,1);
            n4 = segments(j,2);
            flag3 = segments(j,15);
            flag4 = segments(j,16);
            flag_rem = [flag1,flag2,flag3,flag4];
            
          if (1) % treat connected and disconnected segments in the same way
            [f11,f22,f33,f44]=RemoteNodeForce_aniso_quad(...
                           x1,x2,x3,x4,...
                           b1,b2,a,C,tol, ...
                           [flag1,flag2,flag3,flag4], 1); %no trans
          end
          
          if (0) % handle connected segments differently (even slower)
            ntrans = 2;
            if n1 == n3
                [f22,f11,f33,f44]=RemoteNodeForce_aniso_quad(...
                           x2,x2,x3,x4,...
                          -b1,b2,a,C,tol, ...
                           [flag2,flag1,flag3,flag4], ntrans);
            elseif n1 == n4
                [f22,f11,f44,f33]=RemoteNodeForce_aniso_quad(...
                           x2,x1,x4,x3,...
                          -b1,-b2,a,C,tol,  ...
                           [flag2,flag1,flag4,flag3], ntrans);
            elseif n2 == n3
                [f11,f22,f33,f44]=RemoteNodeForce_aniso_quad(...
                           x1,x2,x3,x4,...
                           b1,b2,a,C,tol,  ...
                           [flag1,flag2,flag3,flag4], ntrans);
            elseif n2 == n4
                [f11,f22,f44,f33]=RemoteNodeForce_aniso_quad(...
                           x1,x2,x4,x3,...
                           b1,-b2,a,C,tol,  ...
                           [flag1,flag2,flag4,flag3], ntrans);
            else                
                [f11,f22,f33,f44]=RemoteNodeForce_aniso_quad(...
                           x1,x2,x3,x4,...
                           b1,b2,a,C,tol, ...
                           [flag1,flag2,flag3,flag4], 1); %no trans
            end
          end
         
            f0(i,:) = f0(i,:) + f11;
            f1(i,:) = f1(i,:) + f22;
            f0(j,:) = f0(j,:) + f33;
            f1(j,:) = f1(j,:) + f44;
        end
    end
else
    disp('*** segforcevec.m: linkid neq 0 ***');
    pause;
    tmp=segments(1,:);
    segments(1,:)=segments(linkid,:);
    segments(linkid,:)=tmp;
    f0=zeros(1,3);
    f1=zeros(1,3);
    k=1;
    b1=segments(1,3:5);
    x1=segments(1,6:8);
    x2=segments(1,9:11);
    flag1 = segments(1,15);
    flag2 = segments(1,16);
    n1 = segments(1,1);
    n2 = segments(1,2);
    for j=2:lseg
        b2=segments(j,3:5);
        x3=segments(j,6:8);
        x4=segments(j,9:11);
        flag3 = segments(j,15);
        flag4 = segments(j,16);
        flag_rem = [flag1,flag2,flag3,flag4];
        n3 = segments(j,1);
        n4 = segments(j,2);
        
        % treat connected and disconnected segments in the same way
        [f11,f22,f33,f44]=RemoteNodeForce_aniso_quad(...
                           x1,x2,x3,x4,...
                           b1,b2,a,C,rtol, ...
                           [flag1,flag2,flag3,flag4], 1); %no trans
        
        f0(1,:) = f0(i,:) + f11;
        f1(1,:) = f1(i,:) + f22;
    end
end