function sigma_seg_WS=stress_seg_WS(A,B,P,b,C,meth,a,Nint)
%compute the stress field of a dislocation segment AB at P
%using Willis-Steeds-Lothe formula

%A,B: (1x3) row vector, the starting and ending point of a segment
%P: (1x3) field point
%b:(1x3) Burgers vector
%C: (3x3x3x3, rank-4 tensor) elastic constant tensor
%meth: method used to compute QBS, 'm' or 'i'
%a: dislocation core radius

%meth and a must be specified
%Nint is only valid when meth == 'i'

%Highest priority given to a = 0 case
if a == 0
    if(~exist('Nint'))
        sigma_seg_WS=stress_seg_WS_main(A,B,P,b,C,meth,a);return
    else
        sigma_seg_WS=stress_seg_WS_main(A,B,P,b,C,'i',a,Nint);return
    end
end

%Step 1 Determine if it is collinear case
eps = 1e-6; 

xi=(B-A)/norm(B-A); %segment orientation
D = dot((P-A),xi)*xi + A;
h = norm(P-D);

if (h < eps)
    %collinear case
    if(~exist('Nint'))
        sigma_seg_WS = stress_co(A,B,P,b,C,meth,a);return
    else
        sigma_seg_WS = stress_co(A,B,P,b,C,'i',a,Nint);return
    end
end

%Step 2 Truncate the segment for nonlinear cases

if abs(h-a)<=eps
    h = a;
end

if(~exist('Nint'))
    if h>=a
        sigma_seg_WS=stress_seg_WS_main(A,B,P,b,C,meth,a);
    else
        B1 = D - sqrt(a^2-h^2)*xi;
        A2 = D + sqrt(a^2-h^2)*xi;
        if dot((A2-A),xi) < 0 || dot((B1-B),xi) > 0
            sigma_seg_WS = stress_seg_WS_main(A,B,P,b,C,meth,a);
        elseif dot((B1-A),xi) < 0 && dot((A2-A),xi) > 0
            sigma_seg_WS = stress_seg_WS_main(A2,B,P,b,C,meth,a);
        elseif dot((B1-B),xi) < 0 && dot((A2-B),xi) > 0
            sigma_seg_WS = stress_seg_WS_main(A,B1,P,b,C,meth,a);
        else
            sigma_seg_WS=stress_seg_WS_main(A,B1,P,b,C,meth,a)+...
                stress_seg_WS_main(A2,B,P,b,C,meth,a);
        end
    end
else
    if h>=a
        sigma_seg_WS=stress_seg_WS_main(A,B,P,b,C,'i',a,Nint);
    else
        B1 = D - sqrt(a^2-h^2)*xi;
        A2 = D + sqrt(a^2-h^2)*xi;
        if dot((A2-A),xi) < 0 || dot((B1-B),xi) > 0
            sigma_seg_WS = stress_seg_WS_main(A,B,P,b,C,'i',a,Nint);
        elseif dot((B1-A),xi) < 0 && dot((A2-A),xi) > 0
            sigma_seg_WS = stress_seg_WS_main(A2,B,P,b,C,'i',a,Nint);
        elseif dot((B1-B),xi) < 0 && dot((A2-B),xi) > 0
            sigma_seg_WS = stress_seg_WS_main(A,B1,P,b,C,'i',a,Nint);
        else
            sigma_seg_WS=stress_seg_WS_main(A,B1,P,b,C,'i',a,Nint)+...
                stress_seg_WS_main(A2,B,P,b,C,'i',a,Nint);
        end
    end
end

%==================================================

function sigma_seg_WS=stress_seg_WS_main(A,B,P,b,C,meth, a, Nint) 
%compute the stress field of a dislocation segment
%Willis-Steeds-Lothe formula
%not a collinear case

%Step1 Compute ABP plane normal n
eps = 1e-6;
xi = (B-A)/norm(B-A);
D = dot((P-A),xi)*xi + A;
h = norm(P-D);

if h < eps
    if(exist('Nint'))
        sigma_seg_WS=stress_co(A,B,P,b,C,'i',a, Nint);return
    else
        sigma_seg_WS=stress_co(A,B,P,b,C,meth,a);return
    end
end

ev = (D-P)/h;
n = cross(xi,ev);

%Step2 Calculate Q B S matrices
tau1=(P-A)/norm(P-A);
tau2=(P-B)/norm(P-B);

if meth == 'm'
    [Q1,B1,S1] = stroh_m(tau1, C);
    [Q2,B2,S2] = stroh_m(tau2, C);
elseif exist('Nint')
    [Q1,B1,S1] = stroh_i(tau1, C,Nint);
    [Q2,B2,S2] = stroh_i(tau2, C,Nint);
else
    [Q1,B1,S1] = stroh_i(tau1, C);
    [Q2,B2,S2] = stroh_i(tau2, C);
end

%Step3 Prepare ( ) matrices
m1=cross(n,tau1); m2=cross(n,tau2);
z3=zeros(3); 
nn=z3;nm1=zeros(3); nm2=zeros(3);
%
 for j=1:3
    for k=1:3
        for i=1:3
            for l=1:3
                nn(j,k)=nn(j,k)+n(i)*C(i,j,k,l)*n(l);
                nm1(j,k)=nm1(j,k)+n(i)*C(i,j,k,l)*m1(l);
                nm2(j,k)=nm2(j,k)+n(i)*C(i,j,k,l)*m2(l);
            end
        end
    end
end
%
nn_1=inv(nn);

%Step4 Compute stress field

%define alternating tensor
epsilon=zeros(3,3,3);
epsilon(1,2,3)=1;
epsilon(2,3,1)=1;
epsilon(3,1,2)=1;
epsilon(1,3,2)=-1;
epsilon(2,1,3)=-1;
epsilon(3,2,1)=-1;

%intermediate matrices
M2=nn_1*nm2*Q2+nn_1*S2' ;
M1=nn_1*nm1*Q1+nn_1*S1' ;

%Calculate strain field
strain=z3;
for p=1:3
    for s=1:3
        for j=1:3
            for q=1:3
                for i=1:3
                    for k=1:3
                        for l=1:3
                            strain(p,s)=strain(p,s)+epsilon(j,s,q)*b(i)*C(i,j,k,l)*xi(q)*(-m2(l)*Q2(p,k)+...
                                n(l)*M2(p,k)+m1(l)*Q1(p,k)-n(l)*M1(p,k))/4/pi/h;
                        end
                    end
                end
            end
        end
    end
end

%Calculate stress field
sigma_seg_WS=z3;
for i=1:3
    for j=1:3
        for k=1:3
            for l=1:3
                sigma_seg_WS(i,j)=sigma_seg_WS(i,j)+C(i,j,k,l)*strain(k,l);
            end
        end
    end
end